"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseFunction = exports.parseMethodParams = exports.parseFunctionParts = void 0;
var typescript_1 = require("typescript");
var DefaultDeclaration_1 = require("../declarations/DefaultDeclaration");
var FunctionDeclaration_1 = require("../declarations/FunctionDeclaration");
var ParameterDeclaration_1 = require("../declarations/ParameterDeclaration");
var TypescriptGuards_1 = require("../type-guards/TypescriptGuards");
var identifier_parser_1 = require("./identifier-parser");
var parse_utilities_1 = require("./parse-utilities");
var variable_parser_1 = require("./variable-parser");
/**
 * Parse the parts of a function. All functions / methods contain various information about used variables
 * and parameters.
 *
 * @export
 * @param {Resource} resource
 * @param {(TshConstructor | TshMethod | TshFunction)} parent
 * @param {Node} node
 */
function parseFunctionParts(resource, parent, node) {
    for (var _i = 0, _a = node.getChildren(); _i < _a.length; _i++) {
        var child = _a[_i];
        switch (child.kind) {
            case typescript_1.SyntaxKind.Identifier:
                (0, identifier_parser_1.parseIdentifier)(resource, child);
                break;
            case typescript_1.SyntaxKind.VariableStatement:
                (0, variable_parser_1.parseVariable)(parent, child);
                break;
            default:
                break;
        }
        parseFunctionParts(resource, parent, child);
    }
}
exports.parseFunctionParts = parseFunctionParts;
/**
 * Parse method parameters.
 *
 * @export
 * @param {(FunctionDeclaration | MethodDeclaration | MethodSignature)} node
 * @returns {TshParameter[]}
 */
function parseMethodParams(node) {
    return node.parameters.reduce(function (all, cur) {
        var params = all;
        if ((0, TypescriptGuards_1.isIdentifier)(cur.name)) {
            params.push(new ParameterDeclaration_1.ParameterDeclaration(cur.name.text, (0, parse_utilities_1.getNodeType)(cur.type), cur.getStart(), cur.getEnd()));
        }
        else if ((0, TypescriptGuards_1.isObjectBindingPattern)(cur.name)) {
            var elements = cur.name.elements;
            var types_1 = [];
            var boundParam = new ParameterDeclaration_1.ObjectBoundParameterDeclaration(cur.getStart(), cur.getEnd());
            if (cur.type && (0, typescript_1.isTypeReferenceNode)(cur.type)) {
                boundParam.typeReference = (0, parse_utilities_1.getNodeType)(cur.type);
            }
            else if (cur.type && (0, typescript_1.isTypeLiteralNode)(cur.type)) {
                types_1 = cur.type.members
                    .filter(function (member) { return (0, TypescriptGuards_1.isPropertySignature)(member); })
                    .map(function (signature) { return (0, parse_utilities_1.getNodeType)(signature.type); });
            }
            boundParam.parameters = elements.map(function (bindingElement, index) {
                return new ParameterDeclaration_1.ParameterDeclaration(bindingElement.name.getText(), types_1[index], bindingElement.getStart(), bindingElement.getEnd());
            });
            params.push(boundParam);
        }
        else if ((0, TypescriptGuards_1.isArrayBindingPattern)(cur.name)) {
            var elements = cur.name.elements;
            var types_2 = [];
            var boundParam = new ParameterDeclaration_1.ArrayBoundParameterDeclaration(cur.getStart(), cur.getEnd());
            if (cur.type && (0, typescript_1.isTypeReferenceNode)(cur.type)) {
                boundParam.typeReference = (0, parse_utilities_1.getNodeType)(cur.type);
            }
            else if (cur.type && (0, typescript_1.isTupleTypeNode)(cur.type)) {
                // TODO: needs a fix
                // Previous state: types = cur.type.elementTypes.map(type => getNodeType(type));
                types_2 = cur.type.elements.map(function (type) { return (0, parse_utilities_1.getNodeType)(type); });
            }
            boundParam.parameters = elements.map(function (bindingElement, index) {
                return new ParameterDeclaration_1.ParameterDeclaration(bindingElement.getText(), types_2[index], bindingElement.getStart(), bindingElement.getEnd());
            });
            params.push(boundParam);
        }
        return params;
    }, []);
}
exports.parseMethodParams = parseMethodParams;
/**
 * Parses a function into its declaration.
 * Parses the functions sub information like parameters and variables.
 *
 * @export
 * @param {Resource} resource
 * @param {FunctionDeclaration} node
 */
function parseFunction(resource, node) {
    var name = node.name ? node.name.text : (0, parse_utilities_1.getDefaultResourceIdentifier)(resource);
    var func = new FunctionDeclaration_1.FunctionDeclaration(name, (0, parse_utilities_1.isNodeExported)(node), (0, parse_utilities_1.containsModifier)(node, typescript_1.SyntaxKind.AsyncKeyword), (0, parse_utilities_1.getNodeType)(node.type), node.getStart(), node.getEnd());
    if ((0, parse_utilities_1.isNodeDefaultExported)(node)) {
        func.isExported = false;
        resource.declarations.push(new DefaultDeclaration_1.DefaultDeclaration(func.name, resource));
    }
    func.parameters = parseMethodParams(node);
    resource.declarations.push(func);
    parseFunctionParts(resource, func, node);
}
exports.parseFunction = parseFunction;
